(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/animations'), require('@angular/core'), require('@angular/platform-browser'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('ngx-swing', ['exports', '@angular/animations', '@angular/core', '@angular/platform-browser', 'rxjs/operators'], factory) :
    (factory((global['ngx-swing'] = {}),global.ng.animations,global.ng.core,global.ng.platformBrowser,global.rxjs.operators));
}(this, (function (exports,animations,core,platformBrowser,operators) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @enum {string} */
    var Direction = {
        Left: 'Left',
        Right: 'Right',
        Up: 'Up',
        Down: 'Down',
    };
    /**
     * @param {?} offset
     * @return {?}
     */
    function calculateDirection(offset) {
        if (offset.y === 0 || offset.x === 0) {
            throw zeroOffsetError();
        }
        /** @type {?} */
        var horizontalDirection = offset.x < 0 ? Direction.Left : Direction.Right;
        /** @type {?} */
        var verticalDirection = offset.y < 0 ? Direction.Up : Direction.Down;
        return Math.abs(offset.x) > Math.abs(offset.y) ? horizontalDirection : verticalDirection;
    }
    /**
     * @return {?}
     */
    function zeroOffsetError() {
        return new Error("Offset.x and Offset.y must be lower or greater than zero.");
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * @param {?} dimension
     * @param {?} offset
     * @return {?}
     */
    function calculateRotation(dimension, offset) {
        /** @type {?} */
        var horizontalOffset = Math.min(Math.max(offset.x / dimension.width, -1), 1);
        /** @type {?} */
        var verticalOffset = (offset.y > 0 ? 1 : -1) * Math.min(Math.abs(offset.y) / 100, 1);
        return horizontalOffset * verticalOffset;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var SwingCardComponent = /** @class */ (function () {
        function SwingCardComponent(stack, sanitizer) {
            this.stack = stack;
            this.sanitizer = sanitizer;
            this._offset = { x: 0, y: 0 };
            this._rotation = 0;
            this._lastMove = {};
            this._cardState = CardState.free;
            this._animationParams = {};
            this._rotationFactor = 20;
            this._velocityFactor = 5;
            this.cardStateChange = new core.EventEmitter();
            this.offsetStateChange = new core.EventEmitter();
        }
        Object.defineProperty(SwingCardComponent.prototype, "__cardState", {
            get: /**
             * @return {?}
             */ function () {
                return { value: this._cardState, params: this._animationParams };
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SwingCardComponent.prototype, "_transform", {
            get: /**
             * @return {?}
             */ function () {
                return this.sanitizer.bypassSecurityTrustStyle("translate3d(0, 0, 0) translate(" + this._offset.x + "px, " + this._offset.y + "px) rotate(" + this._rotation + "deg)");
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SwingCardComponent.prototype, "_directionX", {
            get: /**
             * @return {?}
             */ function () {
                return this._offset.x === 0 ? undefined : this._offset.x < 0 ? Direction.Left : Direction.Right;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SwingCardComponent.prototype, "_directionY", {
            get: /**
             * @return {?}
             */ function () {
                return this._offset.y === 0 ? undefined : this._offset.y < 0 ? Direction.Up : Direction.Down;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SwingCardComponent.prototype, "_offsetState", {
            get: /**
             * @return {?}
             */ function () {
                return this._offset.y !== 0 && this._offset.x !== 0 && this._cardState === CardState.free ? OffsetState.moving : OffsetState.stable;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(SwingCardComponent.prototype, "cardState", {
            get: /**
             * @return {?}
             */ function () {
                return this._cardState;
            },
            enumerable: true,
            configurable: true
        });
        /**
         * @param {?} event
         * @return {?}
         */
        SwingCardComponent.prototype._panstart = /**
         * @param {?} event
         * @return {?}
         */
            function (event) {
                this._cardState = CardState.free;
                this._move(event);
            };
        /**
         * @param {?} event
         * @return {?}
         */
        SwingCardComponent.prototype._panmove = /**
         * @param {?} event
         * @return {?}
         */
            function (event) {
                this._move(event);
            };
        /**
         * @param {?} event
         * @return {?}
         */
        SwingCardComponent.prototype._panend = /**
         * @param {?} event
         * @return {?}
         */
            function (event) {
                /** @type {?} */
                var lastX = this._lastMove.deltaX || 0;
                /** @type {?} */
                var lastY = this._lastMove.deltaY || 0;
                /** @type {?} */
                var offset = { x: event.deltaX + lastX, y: event.deltaY + lastY };
                /** @type {?} */
                var isThrowOut = this.getThrowOutRange(offset) === 1;
                if (isThrowOut) {
                    this.throwOut(offset);
                }
                else {
                    this.throwIn(offset);
                }
            };
        /**
         * @param {?} event
         * @return {?}
         */
        SwingCardComponent.prototype._move = /**
         * @param {?} event
         * @return {?}
         */
            function (event) {
                /** @type {?} */
                var lastX = this._lastMove.deltaX || 0;
                /** @type {?} */
                var lastY = this._lastMove.deltaY || 0;
                this._offset = { x: event.deltaX + lastX, y: event.deltaY + lastY };
                this._rotation = calculateRotation(this.getDimensions(), this._offset) * this._rotationFactor;
                this._lastMove = event;
                /** @type {?} */
                var isThrowOut = this.getThrowOutRange(this._offset) === 1;
                this.offsetStateChange.emit({
                    offsetState: OffsetState.moving,
                    cardState: isThrowOut ? CardState.throwOut : CardState.throwIn
                });
            };
        /**
         * @param {?} event
         * @return {?}
         */
        SwingCardComponent.prototype._stateAnimationDone = /**
         * @param {?} event
         * @return {?}
         */
            function (event) {
                if (event.toState == CardState.throwOut) {
                    this._offset = { x: this._animationParams.x, y: this._animationParams.y };
                    this._rotation = this._animationParams.rotation;
                }
                else {
                    this._offset = { x: 0, y: 0 };
                    this._rotation = 0;
                }
            };
        /**
         * @param {?} offset
         * @return {?}
         */
        SwingCardComponent.prototype.throwIn = /**
         * @param {?} offset
         * @return {?}
         */
            function (offset) {
                if (offset.y === 0 || offset.x === 0) {
                    throw new Error("Offset.x and Offset.y must be lower or greater than zero.");
                }
                /** @type {?} */
                var verticalAxis = offset.y < 0 ? 1 : -1;
                /** @type {?} */
                var horizontalAxis = offset.x < 0 ? 1 : -1;
                /** @type {?} */
                var rotation = (calculateRotation(this.getDimensions(), offset) * this._rotationFactor) / this._velocityFactor;
                /** @type {?} */
                var verticalVelocity = this._velocityFactor * verticalAxis;
                /** @type {?} */
                var horizontalVelocity = this._velocityFactor * horizontalAxis;
                this._animationParams = { rotation: rotation, verticalVelocity: verticalVelocity, horizontalVelocity: horizontalVelocity };
                this._cardState = CardState.throwIn;
                this.cardStateChange.emit({ state: this._cardState, direction: calculateDirection(offset) });
            };
        /**
         * @param {?} offset
         * @return {?}
         */
        SwingCardComponent.prototype.throwOut = /**
         * @param {?} offset
         * @return {?}
         */
            function (offset) {
                if (offset.y === 0 || offset.x === 0) {
                    throw new Error("Offset.x and Offset.y must be lower or greater than zero.");
                }
                /** @type {?} */
                var rotation = calculateRotation(this.getDimensions(), offset) * this._rotationFactor * 2;
                /** @type {?} */
                var y = offset.y * 2;
                /** @type {?} */
                var x = offset.x * 2;
                this._animationParams = { rotation: rotation, y: y, x: x };
                this._cardState = CardState.throwOut;
                this.cardStateChange.emit({ state: this._cardState, direction: calculateDirection(offset) });
            };
        /**
         * @return {?}
         */
        SwingCardComponent.prototype.getDimensions = /**
         * @return {?}
         */
            function () {
                return this.stack.getDimensions();
            };
        /**
         * @param {?} offset
         * @return {?}
         */
        SwingCardComponent.prototype.getThrowOutRange = /**
         * @param {?} offset
         * @return {?}
         */
            function (offset) {
                /** @type {?} */
                var dimension = this.getDimensions();
                /** @type {?} */
                var horizontalConfidence = Math.min(Math.abs(offset.x) / dimension.width, 1);
                /** @type {?} */
                var verticalConfidence = Math.min(Math.abs(offset.y) / dimension.height, 1);
                return Math.max(horizontalConfidence, verticalConfidence);
            };
        SwingCardComponent.decorators = [
            { type: core.Component, args: [{
                        selector: 'swing-card,[swingCard]',
                        exportAs: 'swingCard',
                        template: "<ng-content></ng-content>",
                        animations: [
                            animations.trigger('cardState', [
                                animations.transition('* => throwOut', animations.animate('500ms cubic-bezier(0.310, 0.440, 0.445, 1.650)', animations.keyframes([
                                    animations.style({ transform: 'translate({{ x }}px, {{ y }}px) rotate({{ rotation }}deg)' })
                                ]))),
                                animations.transition('* => throwIn', animations.animate('500ms cubic-bezier(0.310, 0.440, 0.445, 1.650)', animations.keyframes([
                                    animations.style({
                                        transform: 'translate(0px, 0px) rotate({{rotation}}deg)',
                                        offset: 0.6
                                    }),
                                    animations.style({
                                        transform: 'translate3d({{ horizontalVelocity }}px, {{ verticalVelocity }}px, 0)',
                                        offset: 0.80,
                                    }),
                                ])))
                            ])
                        ]
                    }] }
        ];
        /** @nocollapse */
        SwingCardComponent.ctorParameters = function () {
            return [
                { type: SwingStackDirective, decorators: [{ type: core.Optional }, { type: core.Inject, args: [core.forwardRef(( /**
                                                 * @return {?}
                                                 */function () { return SwingStackDirective; })),] }] },
                { type: platformBrowser.DomSanitizer }
            ];
        };
        SwingCardComponent.propDecorators = {
            __cardState: [{ type: core.HostBinding, args: ['@cardState',] }],
            _transform: [{ type: core.HostBinding, args: ['style.transform',] }],
            _directionX: [{ type: core.HostBinding, args: ['attr.direction-x',] }],
            _directionY: [{ type: core.HostBinding, args: ['attr.direction-y',] }],
            _offsetState: [{ type: core.HostBinding, args: ['attr.offset-state',] }],
            cardState: [{ type: core.Input }],
            cardStateChange: [{ type: core.Output }],
            offsetStateChange: [{ type: core.Output }],
            _panstart: [{ type: core.HostListener, args: ['panstart', ["$event"],] }],
            _panmove: [{ type: core.HostListener, args: ['panmove', ['$event'],] }],
            _panend: [{ type: core.HostListener, args: ['panend', ['$event'],] }],
            _stateAnimationDone: [{ type: core.HostListener, args: ['@cardState.done', ['$event'],] }]
        };
        return SwingCardComponent;
    }());
    /** @enum {string} */
    var CardState = {
        throwIn: 'throwIn',
        throwOut: 'throwOut',
        free: 'free',
    };
    /** @enum {string} */
    var OffsetState = {
        stable: 'stable',
        moving: 'moving',
    };

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var SwingStackDirective = /** @class */ (function () {
        function SwingStackDirective(elementRef) {
            this.elementRef = elementRef;
            this.cards = new core.QueryList();
            this.cardStateChange = new core.EventEmitter();
            this.offsetStateChange = new core.EventEmitter();
            this.dispose = new core.EventEmitter();
        }
        /**
         * @return {?}
         */
        SwingStackDirective.prototype.ngAfterViewInit = /**
         * @return {?}
         */
            function () {
                var _this = this;
                this.cards.changes.subscribe(( /**
                 * @return {?}
                 */function () {
                    _this.dispose.emit();
                    _this.subscribeCardEvents();
                }));
                this.cards.notifyOnChanges();
            };
        /**
         * @return {?}
         */
        SwingStackDirective.prototype.subscribeCardEvents = /**
         * @return {?}
         */
            function () {
                var _this = this;
                this.cards.forEach(( /**
                 * @param {?} card
                 * @return {?}
                 */function (card) {
                    card.offsetStateChange
                        .pipe(operators.takeUntil(_this.dispose))
                        .subscribe(( /**
                 * @param {?} event
                 * @return {?}
                 */function (event) {
                        _this.offsetStateChange.emit(__assign({}, event, { card: card }));
                    }));
                    card.cardStateChange
                        .pipe(operators.takeUntil(_this.dispose))
                        .subscribe(( /**
                 * @param {?} event
                 * @return {?}
                 */function (event) {
                        _this.cardStateChange.emit(__assign({}, event, { card: card }));
                    }));
                }));
            };
        /**
         * @return {?}
         */
        SwingStackDirective.prototype.getDimensions = /**
         * @return {?}
         */
            function () {
                return { height: this.elementRef.nativeElement.offsetHeight, width: this.elementRef.nativeElement.offsetWidth };
            };
        /**
         * @return {?}
         */
        SwingStackDirective.prototype.ngOnDestroy = /**
         * @return {?}
         */
            function () {
                this.dispose.emit();
            };
        SwingStackDirective.decorators = [
            { type: core.Directive, args: [{
                        selector: 'swing-stack,[swingStack]',
                        exportAs: 'swingStack'
                    },] }
        ];
        /** @nocollapse */
        SwingStackDirective.ctorParameters = function () {
            return [
                { type: core.ElementRef }
            ];
        };
        SwingStackDirective.propDecorators = {
            cards: [{ type: core.ContentChildren, args: [core.forwardRef(( /**
                                     * @return {?}
                                     */function () { return SwingCardComponent; })),] }],
            cardStateChange: [{ type: core.Output }],
            offsetStateChange: [{ type: core.Output }]
        };
        return SwingStackDirective;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    var NgxSwingModule = /** @class */ (function () {
        function NgxSwingModule() {
        }
        NgxSwingModule.decorators = [
            { type: core.NgModule, args: [{
                        declarations: [
                            SwingStackDirective,
                            SwingCardComponent
                        ],
                        exports: [
                            SwingStackDirective,
                            SwingCardComponent
                        ]
                    },] }
        ];
        return NgxSwingModule;
    }());

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    exports.NgxSwingModule = NgxSwingModule;
    exports.SwingCardComponent = SwingCardComponent;
    exports.CardState = CardState;
    exports.OffsetState = OffsetState;
    exports.SwingStackDirective = SwingStackDirective;

    Object.defineProperty(exports, '__esModule', { value: true });

})));

//# sourceMappingURL=ngx-swing.umd.js.map