import { animate, keyframes, style, transition, trigger } from '@angular/animations';
import { Component, EventEmitter, forwardRef, HostBinding, HostListener, Inject, Input, Optional, Output, ContentChildren, Directive, ElementRef, QueryList, NgModule } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { takeUntil } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const Direction = {
    Left: 'Left',
    Right: 'Right',
    Up: 'Up',
    Down: 'Down',
};
/**
 * @param {?} offset
 * @return {?}
 */
function calculateDirection(offset) {
    if (offset.y === 0 || offset.x === 0) {
        throw zeroOffsetError();
    }
    /** @type {?} */
    const horizontalDirection = offset.x < 0 ? Direction.Left : Direction.Right;
    /** @type {?} */
    const verticalDirection = offset.y < 0 ? Direction.Up : Direction.Down;
    return Math.abs(offset.x) > Math.abs(offset.y) ? horizontalDirection : verticalDirection;
}
/**
 * @return {?}
 */
function zeroOffsetError() {
    return new Error(`Offset.x and Offset.y must be lower or greater than zero.`);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} dimension
 * @param {?} offset
 * @return {?}
 */
function calculateRotation(dimension, offset) {
    /** @type {?} */
    const horizontalOffset = Math.min(Math.max(offset.x / dimension.width, -1), 1);
    /** @type {?} */
    const verticalOffset = (offset.y > 0 ? 1 : -1) * Math.min(Math.abs(offset.y) / 100, 1);
    return horizontalOffset * verticalOffset;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SwingCardComponent {
    /**
     * @param {?} stack
     * @param {?} sanitizer
     */
    constructor(stack, sanitizer) {
        this.stack = stack;
        this.sanitizer = sanitizer;
        this._offset = { x: 0, y: 0 };
        this._rotation = 0;
        this._lastMove = {};
        this._cardState = CardState.free;
        this._animationParams = {};
        this._rotationFactor = 20;
        this._velocityFactor = 5;
        this.cardStateChange = new EventEmitter();
        this.offsetStateChange = new EventEmitter();
    }
    /**
     * @return {?}
     */
    get __cardState() {
        return { value: this._cardState, params: this._animationParams };
    }
    /**
     * @return {?}
     */
    get _transform() {
        return this.sanitizer.bypassSecurityTrustStyle(`translate3d(0, 0, 0) translate(${this._offset.x}px, ${this._offset.y}px) rotate(${this._rotation}deg)`);
    }
    /**
     * @return {?}
     */
    get _directionX() {
        return this._offset.x === 0 ? undefined : this._offset.x < 0 ? Direction.Left : Direction.Right;
    }
    /**
     * @return {?}
     */
    get _directionY() {
        return this._offset.y === 0 ? undefined : this._offset.y < 0 ? Direction.Up : Direction.Down;
    }
    /**
     * @return {?}
     */
    get _offsetState() {
        return this._offset.y !== 0 && this._offset.x !== 0 && this._cardState === CardState.free ? OffsetState.moving : OffsetState.stable;
    }
    /**
     * @return {?}
     */
    get cardState() {
        return this._cardState;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _panstart(event) {
        this._cardState = CardState.free;
        this._move(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _panmove(event) {
        this._move(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _panend(event) {
        /** @type {?} */
        const lastX = this._lastMove.deltaX || 0;
        /** @type {?} */
        const lastY = this._lastMove.deltaY || 0;
        /** @type {?} */
        const offset = { x: event.deltaX + lastX, y: event.deltaY + lastY };
        /** @type {?} */
        const isThrowOut = this.getThrowOutRange(offset) === 1;
        if (isThrowOut) {
            this.throwOut(offset);
        }
        else {
            this.throwIn(offset);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _move(event) {
        /** @type {?} */
        const lastX = this._lastMove.deltaX || 0;
        /** @type {?} */
        const lastY = this._lastMove.deltaY || 0;
        this._offset = { x: event.deltaX + lastX, y: event.deltaY + lastY };
        this._rotation = calculateRotation(this.getDimensions(), this._offset) * this._rotationFactor;
        this._lastMove = event;
        /** @type {?} */
        const isThrowOut = this.getThrowOutRange(this._offset) === 1;
        this.offsetStateChange.emit({
            offsetState: OffsetState.moving,
            cardState: isThrowOut ? CardState.throwOut : CardState.throwIn
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    _stateAnimationDone(event) {
        if (event.toState == CardState.throwOut) {
            this._offset = { x: this._animationParams.x, y: this._animationParams.y };
            this._rotation = this._animationParams.rotation;
        }
        else {
            this._offset = { x: 0, y: 0 };
            this._rotation = 0;
        }
    }
    /**
     * @param {?} offset
     * @return {?}
     */
    throwIn(offset) {
        if (offset.y === 0 || offset.x === 0) {
            throw new Error(`Offset.x and Offset.y must be lower or greater than zero.`);
        }
        /** @type {?} */
        const verticalAxis = offset.y < 0 ? 1 : -1;
        /** @type {?} */
        const horizontalAxis = offset.x < 0 ? 1 : -1;
        /** @type {?} */
        const rotation = (calculateRotation(this.getDimensions(), offset) * this._rotationFactor) / this._velocityFactor;
        /** @type {?} */
        const verticalVelocity = this._velocityFactor * verticalAxis;
        /** @type {?} */
        const horizontalVelocity = this._velocityFactor * horizontalAxis;
        this._animationParams = { rotation, verticalVelocity, horizontalVelocity };
        this._cardState = CardState.throwIn;
        this.cardStateChange.emit({ state: this._cardState, direction: calculateDirection(offset) });
    }
    /**
     * @param {?} offset
     * @return {?}
     */
    throwOut(offset) {
        if (offset.y === 0 || offset.x === 0) {
            throw new Error(`Offset.x and Offset.y must be lower or greater than zero.`);
        }
        /** @type {?} */
        const rotation = calculateRotation(this.getDimensions(), offset) * this._rotationFactor * 2;
        /** @type {?} */
        const y = offset.y * 2;
        /** @type {?} */
        const x = offset.x * 2;
        this._animationParams = { rotation, y, x };
        this._cardState = CardState.throwOut;
        this.cardStateChange.emit({ state: this._cardState, direction: calculateDirection(offset) });
    }
    /**
     * @return {?}
     */
    getDimensions() {
        return this.stack.getDimensions();
    }
    /**
     * @param {?} offset
     * @return {?}
     */
    getThrowOutRange(offset) {
        /** @type {?} */
        const dimension = this.getDimensions();
        /** @type {?} */
        const horizontalConfidence = Math.min(Math.abs(offset.x) / dimension.width, 1);
        /** @type {?} */
        const verticalConfidence = Math.min(Math.abs(offset.y) / dimension.height, 1);
        return Math.max(horizontalConfidence, verticalConfidence);
    }
}
SwingCardComponent.decorators = [
    { type: Component, args: [{
                selector: 'swing-card,[swingCard]',
                exportAs: 'swingCard',
                template: `<ng-content></ng-content>`,
                animations: [
                    trigger('cardState', [
                        transition('* => throwOut', animate('500ms cubic-bezier(0.310, 0.440, 0.445, 1.650)', keyframes([
                            style({ transform: 'translate({{ x }}px, {{ y }}px) rotate({{ rotation }}deg)' })
                        ]))),
                        transition('* => throwIn', animate('500ms cubic-bezier(0.310, 0.440, 0.445, 1.650)', keyframes([
                            style({
                                transform: 'translate(0px, 0px) rotate({{rotation}}deg)',
                                offset: 0.6
                            }),
                            style({
                                transform: 'translate3d({{ horizontalVelocity }}px, {{ verticalVelocity }}px, 0)',
                                offset: 0.80,
                            }),
                        ])))
                    ])
                ]
            }] }
];
/** @nocollapse */
SwingCardComponent.ctorParameters = () => [
    { type: SwingStackDirective, decorators: [{ type: Optional }, { type: Inject, args: [forwardRef((/**
                     * @return {?}
                     */
                    () => SwingStackDirective)),] }] },
    { type: DomSanitizer }
];
SwingCardComponent.propDecorators = {
    __cardState: [{ type: HostBinding, args: ['@cardState',] }],
    _transform: [{ type: HostBinding, args: ['style.transform',] }],
    _directionX: [{ type: HostBinding, args: ['attr.direction-x',] }],
    _directionY: [{ type: HostBinding, args: ['attr.direction-y',] }],
    _offsetState: [{ type: HostBinding, args: ['attr.offset-state',] }],
    cardState: [{ type: Input }],
    cardStateChange: [{ type: Output }],
    offsetStateChange: [{ type: Output }],
    _panstart: [{ type: HostListener, args: ['panstart', ["$event"],] }],
    _panmove: [{ type: HostListener, args: ['panmove', ['$event'],] }],
    _panend: [{ type: HostListener, args: ['panend', ['$event'],] }],
    _stateAnimationDone: [{ type: HostListener, args: ['@cardState.done', ['$event'],] }]
};
/** @enum {string} */
const CardState = {
    throwIn: 'throwIn',
    throwOut: 'throwOut',
    free: 'free',
};
/** @enum {string} */
const OffsetState = {
    stable: 'stable',
    moving: 'moving',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SwingStackDirective {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.cards = new QueryList();
        this.cardStateChange = new EventEmitter();
        this.offsetStateChange = new EventEmitter();
        this.dispose = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.cards.changes.subscribe((/**
         * @return {?}
         */
        () => {
            this.dispose.emit();
            this.subscribeCardEvents();
        }));
        this.cards.notifyOnChanges();
    }
    /**
     * @return {?}
     */
    subscribeCardEvents() {
        this.cards.forEach((/**
         * @param {?} card
         * @return {?}
         */
        card => {
            card.offsetStateChange
                .pipe(takeUntil(this.dispose))
                .subscribe((/**
             * @param {?} event
             * @return {?}
             */
            event => {
                this.offsetStateChange.emit(Object.assign({}, event, { card: card }));
            }));
            card.cardStateChange
                .pipe(takeUntil(this.dispose))
                .subscribe((/**
             * @param {?} event
             * @return {?}
             */
            event => {
                this.cardStateChange.emit(Object.assign({}, event, { card: card }));
            }));
        }));
    }
    /**
     * @return {?}
     */
    getDimensions() {
        return { height: this.elementRef.nativeElement.offsetHeight, width: this.elementRef.nativeElement.offsetWidth };
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.dispose.emit();
    }
}
SwingStackDirective.decorators = [
    { type: Directive, args: [{
                selector: 'swing-stack,[swingStack]',
                exportAs: 'swingStack'
            },] }
];
/** @nocollapse */
SwingStackDirective.ctorParameters = () => [
    { type: ElementRef }
];
SwingStackDirective.propDecorators = {
    cards: [{ type: ContentChildren, args: [forwardRef((/**
                 * @return {?}
                 */
                () => SwingCardComponent)),] }],
    cardStateChange: [{ type: Output }],
    offsetStateChange: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxSwingModule {
}
NgxSwingModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    SwingStackDirective,
                    SwingCardComponent
                ],
                exports: [
                    SwingStackDirective,
                    SwingCardComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxSwingModule, SwingCardComponent, CardState, OffsetState, SwingStackDirective };

//# sourceMappingURL=ngx-swing.js.map